@@ -26,8 +26,8 @@
 //! This allows us to perform polynomial operations in O(n)
 //! by performing an O(n log n) FFT over such a domain.
 
-use crate::fft::{multicore::Worker, SparsePolynomial};
-use snarkvm_fields::{batch_inversion, FieldParameters, PrimeField};
+use crate::fft::{DomainCoeff, SparsePolynomial};
+use snarkvm_fields::{batch_inversion, FftField, FftParameters};
 use snarkvm_utilities::{errors::SerializationError, serialize::*};
 
 use rand::Rng;
@@ -51,7 +51,7 @@ fn log2(number: usize) -> usize {
 /// only for fields that have a large multiplicative subgroup of size that is
 /// a power-of-2.
 #[derive(Copy, Clone, Hash, Eq, PartialEq, CanonicalSerialize, CanonicalDeserialize)]
-pub struct EvaluationDomain<F: PrimeField> {
+pub struct EvaluationDomain<F: FftField> {
     /// The size of the domain.
     pub size: u64,
     /// `log_2(self.size)`.
@@ -68,20 +68,13 @@ pub struct EvaluationDomain<F: PrimeField> {
     pub generator_inv: F,
 }
 
-impl<F: PrimeField> fmt::Debug for EvaluationDomain<F> {
+impl<F: FftField> fmt::Debug for EvaluationDomain<F> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "Multiplicative subgroup of size {}", self.size)
     }
 }
 
-impl<F: PrimeField> EvaluationDomain<F> {
-    fn calculate_chunk_size(size: usize) -> usize {
-        match size / rayon::current_num_threads() {
-            0 => 1,
-            chunk_size => chunk_size,
-        }
-    }
-
+impl<F: FftField> EvaluationDomain<F> {
     /// Sample an element that is *not* in the domain.
     pub fn sample_element_outside_domain<R: Rng>(&self, rng: &mut R) -> F {
         let mut t = F::rand(rng);
@@ -98,19 +91,19 @@ impl<F: PrimeField> EvaluationDomain<F> {
         let size = num_coeffs.next_power_of_two() as u64;
         let log_size_of_group = size.trailing_zeros();
 
-        if log_size_of_group >= F::Parameters::TWO_ADICITY {
+        // libfqfft uses > https://github.com/scipr-lab/libfqfft/blob/e0183b2cef7d4c5deb21a6eaf3fe3b586d738fe0/libfqfft/evaluation_domain/domains/basic_radix2_domain.tcc#L33
+        if log_size_of_group > F::FftParameters::TWO_ADICITY {
             return None;
         }
 
         // Compute the generator for the multiplicative subgroup.
-        // It should be 2^(log_size_of_group) root of unity.
-        let mut group_gen = F::root_of_unity();
-        for _ in log_size_of_group..F::Parameters::TWO_ADICITY {
-            group_gen.square_in_place();
-        }
+        // It should be the 2^(log_size_of_group) root of unity.
+        let group_gen = F::get_root_of_unity(size as usize)?;
 
-        let size_as_bigint = F::BigInteger::from(size);
-        let size_as_field_element = F::from_repr(size_as_bigint)?;
+        // Check that it is indeed the 2^(log_size_of_group) root of unity.
+        debug_assert_eq!(group_gen.pow([size]), F::one());
+
+        let size_as_field_element = F::from(size);
         let size_inv = size_as_field_element.inverse()?;
 
         Some(EvaluationDomain {
@@ -128,7 +121,7 @@ impl<F: PrimeField> EvaluationDomain<F> {
     /// having `num_coeffs` coefficients.
     pub fn compute_size_of_domain(num_coeffs: usize) -> Option<usize> {
         let size = num_coeffs.next_power_of_two();
-        if size.trailing_zeros() < F::Parameters::TWO_ADICITY {
+        if size.trailing_zeros() <= F::FftParameters::TWO_ADICITY {
             Some(size)
         } else {
             None
@@ -140,73 +133,67 @@ impl<F: PrimeField> EvaluationDomain<F> {
         self.size as usize
     }
 
-    /// Compute a FFT.
-    pub fn fft(&self, coeffs: &[F]) -> Vec<F> {
+    /// Compute an FFT.
+    pub fn fft<T: DomainCoeff<F>>(&self, coeffs: &[T]) -> Vec<T> {
         let mut coeffs = coeffs.to_vec();
         self.fft_in_place(&mut coeffs);
         coeffs
     }
 
-    /// Compute a FFT, modifying the vector in place.
-    pub fn fft_in_place(&self, coeffs: &mut Vec<F>) {
-        coeffs.resize(self.size(), F::zero());
-        best_fft(coeffs, &Worker::new(), self.group_gen, self.log_size_of_group)
+    /// Compute an FFT, modifying the vector in place.
+    pub fn fft_in_place<T: DomainCoeff<F>>(&self, coeffs: &mut Vec<T>) {
+        coeffs.resize(self.size(), T::zero());
+        best_fft(coeffs, self.group_gen, self.log_size_of_group)
     }
 
-    /// Compute a IFFT.
-    pub fn ifft(&self, evals: &[F]) -> Vec<F> {
+    /// Compute an IFFT.
+    pub fn ifft<T: DomainCoeff<F>>(&self, evals: &[T]) -> Vec<T> {
         let mut evals = evals.to_vec();
         self.ifft_in_place(&mut evals);
         evals
     }
 
-    /// Compute a IFFT, modifying the vector in place.
+    /// Compute an IFFT, modifying the vector in place.
     #[inline]
-    pub fn ifft_in_place(&self, evals: &mut Vec<F>) {
-        evals.resize(self.size(), F::zero());
-        best_fft(evals, &Worker::new(), self.group_gen_inv, self.log_size_of_group);
-        cfg_iter_mut!(evals).for_each(|val| *val *= &self.size_inv);
+    pub fn ifft_in_place<T: DomainCoeff<F>>(&self, evals: &mut Vec<T>) {
+        evals.resize(self.size(), T::zero());
+        best_fft(evals, self.group_gen_inv, self.log_size_of_group);
+        cfg_iter_mut!(evals).for_each(|val| *val *= self.size_inv);
     }
 
-    /// Compute a FFT over a coset of the domain.
-    pub fn coset_fft(&self, coeffs: &[F]) -> Vec<F> {
+    /// Compute an FFT over a coset of the domain.
+    pub fn coset_fft<T: DomainCoeff<F>>(&self, coeffs: &[T]) -> Vec<T> {
         let mut coeffs = coeffs.to_vec();
         self.coset_fft_in_place(&mut coeffs);
         coeffs
     }
 
-    /// Compute a FFT over a coset of the domain, modifying the input vector
+    /// Compute an FFT over a coset of the domain, modifying the input vector
     /// in place.
-    pub fn coset_fft_in_place(&self, coeffs: &mut Vec<F>) {
+    pub fn coset_fft_in_place<T: DomainCoeff<F>>(&self, coeffs: &mut Vec<T>) {
         Self::distribute_powers(coeffs, F::multiplicative_generator());
         self.fft_in_place(coeffs);
     }
 
-    /// Compute a IFFT over a coset of the domain.
-    pub fn coset_ifft(&self, evals: &[F]) -> Vec<F> {
+    /// Compute an IFFT over a coset of the domain.
+    pub fn coset_ifft<T: DomainCoeff<F>>(&self, evals: &[T]) -> Vec<T> {
         let mut evals = evals.to_vec();
         self.coset_ifft_in_place(&mut evals);
         evals
     }
 
-    /// Compute a IFFT over a coset of the domain, modifying the input vector in place.
-    pub fn coset_ifft_in_place(&self, evals: &mut Vec<F>) {
+    /// Compute an IFFT over a coset of the domain, modifying the input vector in place.
+    pub fn coset_ifft_in_place<T: DomainCoeff<F>>(&self, evals: &mut Vec<T>) {
         self.ifft_in_place(evals);
         Self::distribute_powers(evals, self.generator_inv);
     }
 
-    fn distribute_powers(coeffs: &mut Vec<F>, g: F) {
-        Worker::new().scope(coeffs.len(), |scope, chunk| {
-            for (i, v) in coeffs.chunks_mut(chunk).enumerate() {
-                scope.spawn(move |_| {
-                    let mut u = g.pow(&[(i * chunk) as u64]);
-                    for v in v.iter_mut() {
-                        *v *= &u;
-                        u *= &g;
-                    }
-                });
-            }
-        });
+    fn distribute_powers<T: DomainCoeff<F>>(coeffs: &mut Vec<T>, g: F) {
+        let mut pow = F::one();
+        coeffs.iter_mut().for_each(|c| {
+            *c *= pow;
+            pow *= &g
+        })
     }
 
     /// Evaluate all the lagrange polynomials defined by this domain at the point
@@ -228,12 +215,12 @@ impl<F: PrimeField> EvaluationDomain<F> {
             }
             u
         } else {
-            let mut l = (t_size - &one) * &self.size_inv;
+            let mut l = (t_size - one) * self.size_inv;
             let mut r = one;
             let mut u = vec![F::zero(); size];
             let mut ls = vec![F::zero(); size];
             for i in 0..size {
-                u[i] = tau - &r;
+                u[i] = tau - r;
                 ls[i] = l;
                 l *= &self.group_gen;
                 r *= &self.group_gen;
@@ -241,7 +228,7 @@ impl<F: PrimeField> EvaluationDomain<F> {
 
             batch_inversion(u.as_mut_slice());
             cfg_iter_mut!(u).zip(ls).for_each(|(tau_minus_r, l)| {
-                *tau_minus_r = l * tau_minus_r;
+                *tau_minus_r = l * *tau_minus_r;
             });
             u
         }
@@ -256,7 +243,7 @@ impl<F: PrimeField> EvaluationDomain<F> {
     /// This evaluates the vanishing polynomial for this domain at tau.
     /// For multiplicative subgroups, this polynomial is `z(X) = X^self.size - 1`.
     pub fn evaluate_vanishing_polynomial(&self, tau: F) -> F {
-        tau.pow(&[self.size]) - &F::one()
+        tau.pow(&[self.size]) - F::one()
     }
 
     /// Return an iterator over the elements of the domain.
@@ -277,11 +264,7 @@ impl<F: PrimeField> EvaluationDomain<F> {
             .inverse()
             .unwrap();
 
-        Worker::new().scope(evals.len(), |scope, chunk| {
-            for evals in evals.chunks_mut(chunk) {
-                scope.spawn(move |_| evals.iter_mut().for_each(|eval| *eval *= &i));
-            }
-        });
+        cfg_iter_mut!(evals).for_each(|eval| *eval *= &i);
     }
 
     /// Given an index which assumes the first elements of this domain are the elements of
@@ -316,15 +299,10 @@ impl<F: PrimeField> EvaluationDomain<F> {
     #[must_use]
     pub fn mul_polynomials_in_evaluation_domain(&self, self_evals: &[F], other_evals: &[F]) -> Vec<F> {
         assert_eq!(self_evals.len(), other_evals.len());
+
         let mut result = self_evals.to_vec();
-        let chunk_size = Self::calculate_chunk_size(self.size());
-        cfg_chunks_mut!(result, chunk_size)
-            .zip(cfg_chunks!(other_evals, chunk_size))
-            .for_each(|(a, b)| {
-                for (a, b) in a.iter_mut().zip(b) {
-                    *a *= b;
-                }
-            });
+        cfg_iter_mut!(result).zip(other_evals).for_each(|(a, b)| *a *= b);
+
         result
     }
 
@@ -372,7 +350,7 @@ impl<F: PrimeField> EvaluationDomain<F> {
         if log_powers.len() <= LOG_ROOTS_OF_UNITY_PARALLEL_SIZE {
             powers[0] = F::one();
             for i in 1..powers.len() {
-                powers[i] = powers[i - 1] * &log_powers[0];
+                powers[i] = powers[i - 1] * log_powers[0];
             }
             return;
         }
@@ -415,23 +393,24 @@ impl<F: PrimeField> EvaluationDomain<F> {
 
 #[allow(unused_variables)]
 #[cfg(not(feature = "parallel"))]
-fn best_fft<F: PrimeField>(a: &mut [F], worker: &Worker, omega: F, log_n: u32) {
+fn best_fft<T: DomainCoeff<F>, F: FftField>(a: &mut [T], omega: F, log_n: u32) {
     serial_radix2_fft(a, omega, log_n);
 }
 
 #[cfg(feature = "parallel")]
-fn best_fft<F: PrimeField>(a: &mut [F], worker: &Worker, omega: F, log_n: u32) {
-    let log_cpus = worker.log_num_cpus();
+fn best_fft<T: DomainCoeff<F>, F: FftField>(a: &mut [T], omega: F, log_n: u32) {
+    let num_cpus = rayon::current_num_threads();
+    let log_cpus = log2_floor(num_cpus);
 
     if log_n <= log_cpus {
-        serial_radix2_fft(a, omega, log_n);
+        serial_radix2_fft::<T, F>(a, omega, log_n);
     } else {
-        parallel_radix2_fft(a, worker, omega, log_n, log_cpus);
+        parallel_radix2_fft::<T, F>(a, omega, log_n, log_cpus);
     }
 }
 
 #[allow(clippy::many_single_char_names)]
-pub(crate) fn serial_radix2_fft<F: PrimeField>(a: &mut [F], omega: F, log_n: u32) {
+pub(crate) fn serial_radix2_fft<T: DomainCoeff<F>, F: FftField>(a: &mut [T], omega: F, log_n: u32) {
     #[inline]
     fn bitreverse(mut n: u32, l: u32) -> u32 {
         let mut r = 0;
@@ -461,11 +440,11 @@ pub(crate) fn serial_radix2_fft<F: PrimeField>(a: &mut [F], omega: F, log_n: u32
             let mut w = F::one();
             for j in 0..m {
                 let mut t = a[(k + j + m) as usize];
-                t *= &w;
+                t *= w;
                 let mut tmp = a[(k + j) as usize];
-                tmp -= &t;
+                tmp -= t;
                 a[(k + j + m) as usize] = tmp;
-                a[(k + j) as usize] += &t;
+                a[(k + j) as usize] += t;
                 w.mul_assign(&w_m);
             }
 
@@ -477,66 +456,52 @@ pub(crate) fn serial_radix2_fft<F: PrimeField>(a: &mut [F], omega: F, log_n: u32
 }
 
 #[cfg(feature = "parallel")]
-pub(crate) fn parallel_radix2_fft<F: PrimeField>(a: &mut [F], worker: &Worker, omega: F, log_n: u32, log_cpus: u32) {
+pub(crate) fn parallel_radix2_fft<T: DomainCoeff<F>, F: FftField>(a: &mut [T], omega: F, log_n: u32, log_cpus: u32) {
     assert!(log_n >= log_cpus);
 
-    let num_cpus = 1 << log_cpus;
-    let log_new_n = log_n - log_cpus;
-    let mut tmp = vec![vec![F::zero(); 1 << log_new_n]; num_cpus];
-    let new_omega = omega.pow(&[num_cpus as u64]);
+    let m = a.len();
+    let num_chunks = 1 << (log_cpus as usize);
+    assert_eq!(m % num_chunks, 0);
+    let m_div_num_chunks = m / num_chunks;
 
-    worker.scope(0, |scope, _| {
-        let a = &*a;
+    let mut tmp = vec![vec![T::zero(); m_div_num_chunks]; num_chunks];
+    let new_omega = omega.pow(&[num_chunks as u64]);
+    let new_two_adicity = F::k_adicity(2, m_div_num_chunks);
 
-        for (j, tmp) in tmp.iter_mut().enumerate() {
-            scope.spawn(move |_| {
-                // Shuffle into a sub-FFT
-                let omega_j = omega.pow(&[j as u64]);
-                let omega_step = omega.pow(&[(j as u64) << log_new_n]);
+    tmp.par_iter_mut().enumerate().for_each(|(j, tmp)| {
+        // Shuffle into a sub-FFT
+        let omega_j = omega.pow(&[j as u64]);
+        let omega_step = omega.pow(&[(j * m_div_num_chunks) as u64]);
 
-                let mut elt = F::one();
-                for (i, x) in tmp.iter_mut().enumerate().take(1 << log_new_n) {
-                    for s in 0..num_cpus {
-                        let idx = (i + (s << log_new_n)) % (1 << log_n);
-                        let mut t = a[idx];
-                        t *= &elt;
-                        *x += &t;
-                        elt *= &omega_step;
-                    }
-                    elt *= &omega_j;
-                }
-
-                // Perform sub-FFT
-                serial_radix2_fft(tmp, new_omega, log_new_n);
-            });
+        let mut elt = F::one();
+        for i in 0..m_div_num_chunks {
+            for s in 0..num_chunks {
+                let idx = (i + (s * m_div_num_chunks)) % m;
+                let mut t = a[idx];
+                t *= elt;
+                tmp[i] += t;
+                elt *= &omega_step;
+            }
+            elt *= &omega_j;
         }
+
+        // Perform sub-FFT
+        serial_radix2_fft(tmp, new_omega, new_two_adicity);
     });
 
-    // TODO: does this hurt or help?
-    worker.scope(a.len(), |scope, chunk| {
-        let tmp = &tmp;
-
-        for (idx, a) in a.chunks_mut(chunk).enumerate() {
-            scope.spawn(move |_| {
-                let mut idx = idx * chunk;
-                let mask = (1 << log_cpus) - 1;
-                for a in a {
-                    *a = tmp[idx & mask][idx >> log_cpus];
-                    idx += 1;
-                }
-            });
-        }
-    });
+    a.iter_mut()
+        .enumerate()
+        .for_each(|(i, a)| *a = tmp[i % num_chunks][i / num_chunks]);
 }
 
 /// An iterator over the elements of the domain.
-pub struct Elements<F: PrimeField> {
+pub struct Elements<F: FftField> {
     cur_elem: F,
     cur_pow: u64,
     domain: EvaluationDomain<F>,
 }
 
-impl<F: PrimeField> Iterator for Elements<F> {
+impl<F: FftField> Iterator for Elements<F> {
     type Item = F;
 
     fn next(&mut self) -> Option<F> {
@@ -551,11 +516,32 @@ impl<F: PrimeField> Iterator for Elements<F> {
     }
 }
 
+pub(crate) fn log2_floor(num: usize) -> u32 {
+    assert!(num > 0);
+    let mut pow = 0;
+    while (1 << (pow + 1)) <= num {
+        pow += 1;
+    }
+    pow
+}
+
+#[test]
+fn test_log2_floor() {
+    assert_eq!(log2_floor(1), 0);
+    assert_eq!(log2_floor(2), 1);
+    assert_eq!(log2_floor(3), 1);
+    assert_eq!(log2_floor(4), 2);
+    assert_eq!(log2_floor(5), 2);
+    assert_eq!(log2_floor(6), 2);
+    assert_eq!(log2_floor(7), 2);
+    assert_eq!(log2_floor(8), 3);
+}
+
 #[cfg(test)]
 mod tests {
     use crate::fft::{DensePolynomial, EvaluationDomain};
     use snarkvm_curves::bls12_377::Fr;
-    use snarkvm_fields::{Field, One, PrimeField, Zero};
+    use snarkvm_fields::{FftField, Field, One, Zero};
     use snarkvm_utilities::UniformRand;
 
     use rand::{thread_rng, Rng};
@@ -623,7 +609,7 @@ mod tests {
             // Do lagrange interpolation, and compare against the actual evaluation
             let mut interpolated_evaluation = Fr::zero();
             for i in 0..domain_size {
-                interpolated_evaluation += &(lagrange_coefficients[i] * &polynomial_evaluations[i]);
+                interpolated_evaluation += lagrange_coefficients[i] * polynomial_evaluations[i];
             }
             assert_eq!(actual_evaluations, interpolated_evaluation);
         }
@@ -689,7 +675,7 @@ mod tests {
             let polynomial_evaluations = domain.fft(&random_polynomial.coeffs);
             let polynomial_coset_evaluations = domain.coset_fft(&random_polynomial.coeffs);
             for (i, x) in domain.elements().enumerate() {
-                let coset_x = Fr::multiplicative_generator() * &x;
+                let coset_x = Fr::multiplicative_generator() * x;
 
                 assert_eq!(polynomial_evaluations[i], random_polynomial.evaluate(x));
                 assert_eq!(polynomial_coset_evaluations[i], random_polynomial.evaluate(coset_x));
